package gregapi.recipes;

import static gregapi.data.CS.F;
import gregapi.code.ICondition;
import gregapi.oredict.OreDictItemData;
import gregapi.oredict.OreDictMaterial;
import gregapi.oredict.OreDictPrefix;
import gregapi.util.CR;
import gregapi.util.OM;
import gregapi.util.ST;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;

/**
 * @author Gregorius Techneticies
 */
public class AdvancedCrafting1ToY implements ICraftingRecipeGT {
	public final ICondition mCondition;
	public final OreDictPrefix mInput, mOutput;
	public final boolean mAutoCraftable;
	public final int mOutputCount, mEmpty;
	
	public AdvancedCrafting1ToY(OreDictPrefix aInput, OreDictPrefix aOutput, int aOutputCount, boolean aAutoCraftable) {this(aInput, aOutput, aOutputCount, aAutoCraftable, ICondition.TRUE);}
	public AdvancedCrafting1ToY(OreDictPrefix aInput, OreDictPrefix aOutput, int aOutputCount, boolean aAutoCraftable, ICondition aCondition) {
		mAutoCraftable = aAutoCraftable;
		mCondition = aCondition;
		mInput = aInput;
		mEmpty = mInput.mShapelessManagersSingle.size();
		mOutput = aOutput;
		mOutputCount = aOutputCount;
		mInput.mShapelessManagersSingle.add(this);
		
		
		for (OreDictMaterial tMaterial : mInput.mRegisteredMaterials) if (mCondition.isTrue(tMaterial)) {
			ItemStack tStack = mInput.mat(tMaterial, 1);
			if (ST.valid(tStack)) CR.remove(tStack);
		}
	}
	
	@Override
	public boolean matches(InventoryCrafting aGrid, World aWorld) {
		ItemStack tStack = null;
		OreDictMaterial rMaterial = null;
		
		int tInventorySize = aGrid.getSizeInventory(), tCounter = 0, tEmpty = 0;
		if (tInventorySize < 1+mEmpty) return F;
		for (int i = 0; i < tInventorySize; i++) {
			tStack = aGrid.getStackInSlot(i);
			if (ST.valid(tStack)) {
				OreDictItemData tData = OM.anydata_(tStack);
				if (tData == null || tData.mPrefix != mInput) return F;
				if (rMaterial == null) rMaterial = tData.mMaterial.mMaterial; else if (rMaterial != tData.mMaterial.mMaterial) return F;
				tCounter++;
				continue;
			}
			if (tCounter == 0) tEmpty++;
			if (i-tCounter+1+mEmpty-tEmpty >= tInventorySize) return F;
		}
		return rMaterial != null && tCounter == 1 && tEmpty % mInput.mShapelessManagersSingle.size() == mEmpty && hasOutputFor(rMaterial);
	}
	
	@Override
	public ItemStack getCraftingResult(InventoryCrafting aGrid) {
		for (int i = 0, j = aGrid.getSizeInventory(); i < j; i++) {
			OreDictItemData tData = OM.anydata(aGrid.getStackInSlot(i));
			if (tData == null || tData.mMaterial == null) continue;
			return mOutput.mat(tData.mMaterial.mMaterial, mOutputCount);
		}
		return ERROR_OUTPUT.copy();
	}
	
	public boolean hasOutputFor(OreDictMaterial aMaterial) {
		return ST.valid(mOutput.mat(aMaterial, mOutputCount)) && mCondition.isTrue(aMaterial);
	}
	
	@Override public boolean isRemovableByGT() {return F;}
	@Override public boolean isAutocraftableByGT() {return mAutoCraftable;}
	@Override public int getRecipeSize() {return 1;}
	@Override public ItemStack getRecipeOutput() {return ERROR_OUTPUT.copy();}
}